//
// Rippled water fragment shader
//

#define	EPS	0.01

uniform samplerCube envMap;
uniform sampler2D	h4Map;
uniform sampler2D	h4Map2;
uniform sampler2D	foamMap;
uniform float		time;
uniform float		amplitude2;

varying vec3  n;
varying vec3  e;
varying	vec3  p;


void main (void)
{
	const vec4	c2  = 2.2 * vec4 ( 0.03, 0.15, 0.125, 1.0 );
	const vec4	c1  = 1.6 * vec4 ( 0.03, 0.2,  0.07, 1.0 );
	const vec3	vel = vec3 ( 0.0612149, 0.04101, 0.071109 ) * 0.5;
	
    					// Compute reflection vector
	vec3	arg = 0.2*p + vel * time;
  	vec3	en  = normalize ( e );
	float	t   = fract     ( time );
	vec3	h   = mix       ( texture2D ( h4Map, arg.xy ), texture2D ( h4Map2, arg.xy ), t ).rgb;
  	vec3	nn  = normalize ( n + amplitude2 * h );
    vec3	r   = reflect   ( en, nn );
    
    					// Do a lookup into the environment map.
    vec4 envColor  = textureCube ( envMap, r );
	vec4 foamColor = texture2D   ( foamMap, p.xy + vel.xy * time );

    					// calc fresnels term.  This allows a view dependant
    					// blend of reflection/refraction
    float fresnel = clamp ( abs ( dot ( en, nn ) ), 0.1, 0.9 );

	vec4	color = mix ( envColor, mix ( c1, c2, abs ( dot ( en, nn ) ) ), fresnel );
	
	if ( 2.0 + p.z > 0.3 )	// add foam
		color = mix ( color, foamColor, min ( 2.0+p.z-0.3, 0.2 ) );

	gl_FragColor = color + 30.0*pow ( color * 1.15, vec4 ( 16.0 ) );
}